/*
Copyright (C) 2005-2007 Michael Keith, University Of Manchester

email: mkeith@pulsarastronomy.net
www  : www.pulsarastronomy.net/wiki/Software/PulsarHunter

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
/*
 * Colourmaps.java
 *
 * Created on 24 May 2005, 18:15
 */

package pulsarhunter.jreaper;

import java.awt.Color;
import java.io.Serializable;

/**
 *
 * @author mkeith
 */
public class Colourmap implements Serializable{
    
    
    
    public static Color getFromText(String text){
        if(text.startsWith("#")){
            try{
                int r = Integer.parseInt(text.substring(1,3), 16);
                int g = Integer.parseInt(text.substring(3,5), 16);
                int b = Integer.parseInt(text.substring(5,7), 16);
                return new Color(r,g,b);
            }catch(Exception e){
                return null;
            }
        } else {
            if(text.trim().startsWith("light")) return getFromText(text.trim().substring(5)).brighter();
            if(text.trim().startsWith("dark")) return getFromText(text.trim().substring(4)).darker();
            if(text.trim().equalsIgnoreCase("white"))return Color.white;
            if(text.trim().equalsIgnoreCase("red"))return Color.red;
            if(text.trim().equalsIgnoreCase("green"))return Color.green;
            if(text.trim().equalsIgnoreCase("blue"))return Color.blue;
            if(text.trim().equalsIgnoreCase("black"))return Color.black;
            if(text.trim().equalsIgnoreCase("cyan"))return Color.CYAN;
            if(text.trim().equalsIgnoreCase("gray"))return Color.GRAY;
            if(text.trim().equalsIgnoreCase("magenta"))return Color.magenta;
            if(text.trim().equalsIgnoreCase("orange"))return Color.ORANGE;
            if(text.trim().equalsIgnoreCase("pink"))return Color.pink;
            if(text.trim().equalsIgnoreCase("yellow"))return Color.YELLOW;            
        }
        return null;
    }
    
//        public static Colourmap getCurrentSTDcolormap(){
//            return std_defaultColmap;
//        }
//    
//    public static Colourmap getCurrentZcolormap(){
//        
//        return zCol;
//    }
//    
//    public static Colourmap getCurrentGrayColormap(){
//        
//        return gCol;
//    }
//    
//    public static Colourmap[] getZcolourmaps(){
//        return zMaps;
//    }
//    
//    public static Colourmap[] getGrayscaleColormaps(){
//        return gMaps;
//    }
//    
//    
//    public static void useZcolormap(Colourmap colmap){
//        zCol = colmap;
//    }
//    
//    
//    public static void useGraycolormap(Colourmap colmap){
//        gCol = colmap;
//    }
//    
//    public static Color getCurrentMainViewColour(int i){
//        return stdColmap[i%stdColmap.length];
//    }
//    public static void setCurrentMainViewColour(int i, Color col){
//        stdColmap[i%stdColmap.length] = col;
//    }
    
    
    private Color[] cols;
    private int[][] gCols;
    private boolean isGray;
    private String name;
    /** Creates a new instance of Colourmaps */
    public Colourmap(Color[] cols,String name) {
        isGray = false;
        this.cols = cols;
        this.name = name;
    }
    
    public Colourmap(int[][] cols,String name) {
        isGray = true;
        this.gCols = cols;
        this.name = name;
    }
    
    public String toString(){
        return name;
    }
    
    public int[][] getGCols(){
        return gCols;
    }
    
    public Color[] getCols(){
        return cols;
    }
    
    public boolean equals(Object o){
        if(o instanceof Colourmap){
            return o.toString().equals(this.toString());
        } else return false;
    }
    
    
    
    
    // ------- RAW MAPS BELOW HERE -----  //
    
    public static final int[][] INVERSECOLOURMAP = {
        {255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160,159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144,143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128,127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112,111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96,95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80,79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64,63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48,47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32,31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,},
        {255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160,159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144,143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128,127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112,111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96,95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80,79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64,63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48,47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32,31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,},
        {255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160,159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144,143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128,127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112,111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96,95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80,79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64,63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48,47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32,31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,}
    };
    public static final Colourmap gray_inverse = new Colourmap(INVERSECOLOURMAP,"Linear");
    
    public static final int[][] LINEARCHOP = {
        {254,253,252,251,249,248,247,246,244,243,242,240,239,238,237,235,234,233,232,230,229,228,226,225,224,223,221,220,219,218,216,215,214,212,211,210,209,207,206,205,204,202,201,200,198,197,196,195,193,192,191,189,188,187,186,184,183,182,181,179,178,177,175,174,173,172,170,169,168,167,165,164,163,161,160,159,158,156,155,154,153,151,150,149,147,146,145,144,142,141,140,138,137,136,135,133,132,131,130,128,127,126,124,123,122,121,119,118,117,116,114,113,112,110,109,108,107,105,104,103,102,100,99,98,96,95,94,93,91,90,89,87,86,85,84,82,81,80,79,77,76,75,73,72,71,70,68,67,66,65,63,62,61,59,58,57,56,54,53,52,51,49,48,47,45,44,43,42,40,39,38,36,35,34,33,31,30,29,28,26,25,24,22,21,20,19,17,16,15,14,12,11,10,8,7,6,5,3,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
        {254,253,252,251,249,248,247,246,244,243,242,240,239,238,237,235,234,233,232,230,229,228,226,225,224,223,221,220,219,218,216,215,214,212,211,210,209,207,206,205,204,202,201,200,198,197,196,195,193,192,191,189,188,187,186,184,183,182,181,179,178,177,175,174,173,172,170,169,168,167,165,164,163,161,160,159,158,156,155,154,153,151,150,149,147,146,145,144,142,141,140,138,137,136,135,133,132,131,130,128,127,126,124,123,122,121,119,118,117,116,114,113,112,110,109,108,107,105,104,103,102,100,99,98,96,95,94,93,91,90,89,87,86,85,84,82,81,80,79,77,76,75,73,72,71,70,68,67,66,65,63,62,61,59,58,57,56,54,53,52,51,49,48,47,45,44,43,42,40,39,38,36,35,34,33,31,30,29,28,26,25,24,22,21,20,19,17,16,15,14,12,11,10,8,7,6,5,3,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},
        {254,253,252,251,249,248,247,246,244,243,242,240,239,238,237,235,234,233,232,230,229,228,226,225,224,223,221,220,219,218,216,215,214,212,211,210,209,207,206,205,204,202,201,200,198,197,196,195,193,192,191,189,188,187,186,184,183,182,181,179,178,177,175,174,173,172,170,169,168,167,165,164,163,161,160,159,158,156,155,154,153,151,150,149,147,146,145,144,142,141,140,138,137,136,135,133,132,131,130,128,127,126,124,123,122,121,119,118,117,116,114,113,112,110,109,108,107,105,104,103,102,100,99,98,96,95,94,93,91,90,89,87,86,85,84,82,81,80,79,77,76,75,73,72,71,70,68,67,66,65,63,62,61,59,58,57,56,54,53,52,51,49,48,47,45,44,43,42,40,39,38,36,35,34,33,31,30,29,28,26,25,24,22,21,20,19,17,16,15,14,12,11,10,8,7,6,5,3,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}
    };
    public static final Colourmap gray_linearchop = new Colourmap(LINEARCHOP,"Linear Chopped");
    
    public static final int[][] SQUAREMAP = {
        {255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,254,254,254,254,254,254,253,253,253,253,253,252,252,252,252,251,251,251,251,250,250,250,250,249,249,249,248,248,248,247,247,246,246,246,245,245,244,244,244,243,243,242,242,241,241,240,240,239,239,238,238,237,237,236,236,235,235,234,233,233,232,232,231,230,230,229,228,228,227,226,226,225,224,224,223,222,222,221,220,219,219,218,217,216,215,215,214,213,212,211,211,210,209,208,207,206,205,205,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,176,175,174,173,172,171,170,168,167,166,165,164,162,161,160,159,158,156,155,154,153,151,150,149,147,146,145,143,142,141,139,138,137,135,134,133,131,130,128,127,126,124,123,121,120,118,117,115,114,112,111,109,108,106,105,103,102,100,99,97,95,94,92,91,89,87,86,84,83,81,79,78,76,74,73,71,69,67,66,64,62,60,59,57,55,53,52,50,48,46,44,43,41,39,37,35,33,31,30,28,26,24,22,20,18,16,14,12,10,8,6,4,2,0,},
        {255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,254,254,254,254,254,254,253,253,253,253,253,252,252,252,252,251,251,251,251,250,250,250,250,249,249,249,248,248,248,247,247,246,246,246,245,245,244,244,244,243,243,242,242,241,241,240,240,239,239,238,238,237,237,236,236,235,235,234,233,233,232,232,231,230,230,229,228,228,227,226,226,225,224,224,223,222,222,221,220,219,219,218,217,216,215,215,214,213,212,211,211,210,209,208,207,206,205,205,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,176,175,174,173,172,171,170,168,167,166,165,164,162,161,160,159,158,156,155,154,153,151,150,149,147,146,145,143,142,141,139,138,137,135,134,133,131,130,128,127,126,124,123,121,120,118,117,115,114,112,111,109,108,106,105,103,102,100,99,97,95,94,92,91,89,87,86,84,83,81,79,78,76,74,73,71,69,67,66,64,62,60,59,57,55,53,52,50,48,46,44,43,41,39,37,35,33,31,30,28,26,24,22,20,18,16,14,12,10,8,6,4,2,0,},
        {255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,254,254,254,254,254,254,253,253,253,253,253,252,252,252,252,251,251,251,251,250,250,250,250,249,249,249,248,248,248,247,247,246,246,246,245,245,244,244,244,243,243,242,242,241,241,240,240,239,239,238,238,237,237,236,236,235,235,234,233,233,232,232,231,230,230,229,228,228,227,226,226,225,224,224,223,222,222,221,220,219,219,218,217,216,215,215,214,213,212,211,211,210,209,208,207,206,205,205,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,176,175,174,173,172,171,170,168,167,166,165,164,162,161,160,159,158,156,155,154,153,151,150,149,147,146,145,143,142,141,139,138,137,135,134,133,131,130,128,127,126,124,123,121,120,118,117,115,114,112,111,109,108,106,105,103,102,100,99,97,95,94,92,91,89,87,86,84,83,81,79,78,76,74,73,71,69,67,66,64,62,60,59,57,55,53,52,50,48,46,44,43,41,39,37,35,33,31,30,28,26,24,22,20,18,16,14,12,10,8,6,4,2,0,}
    };
    public static final Colourmap gray_quadratic = new Colourmap(SQUAREMAP,"Quadratic");
    
    private static final Colourmap[] gMaps = {gray_inverse,gray_linearchop,gray_quadratic};
    
    public static final Color[] greyScaleZAxisRev ={
        new Color(0x000000),
                new Color(0x010101),
                new Color(0x020202),
                new Color(0x030303),
                new Color(0x040404),
                new Color(0x050505),
                new Color(0x060606),
                new Color(0x070707),
                new Color(0x080808),
                new Color(0x090909),
                new Color(0x0a0a0a),
                new Color(0x0b0b0b),
                new Color(0x0c0c0c),
                new Color(0x0d0d0d),
                new Color(0x0e0e0e),
                new Color(0x0f0f0f),
                new Color(0x101010),
                new Color(0x111111),
                new Color(0x121212),
                new Color(0x131313),
                new Color(0x141414),
                new Color(0x151515),
                new Color(0x161616),
                new Color(0x171717),
                new Color(0x181818),
                new Color(0x191919),
                new Color(0x1a1a1a),
                new Color(0x1b1b1b),
                new Color(0x1c1c1c),
                new Color(0x1d1d1d),
                new Color(0x1e1e1e),
                new Color(0x1f1f1f),
                new Color(0x202020),
                new Color(0x212121),
                new Color(0x222222),
                new Color(0x232323),
                new Color(0x242424),
                new Color(0x252525),
                new Color(0x262626),
                new Color(0x272727),
                new Color(0x282828),
                new Color(0x292929),
                new Color(0x2a2a2a),
                new Color(0x2b2b2b),
                new Color(0x2c2c2c),
                new Color(0x2d2d2d),
                new Color(0x2e2e2e),
                new Color(0x2f2f2f),
                new Color(0x303030),
                new Color(0x313131),
                new Color(0x323232),
                new Color(0x333333),
                new Color(0x343434),
                new Color(0x353535),
                new Color(0x363636),
                new Color(0x373737),
                new Color(0x383838),
                new Color(0x393939),
                new Color(0x3a3a3a),
                new Color(0x3b3b3b),
                new Color(0x3c3c3c),
                new Color(0x3d3d3d),
                new Color(0x3e3e3e),
                new Color(0x3f3f3f),
                new Color(0x404040),
                new Color(0x414141),
                new Color(0x424242),
                new Color(0x434343),
                new Color(0x444444),
                new Color(0x454545),
                new Color(0x464646),
                new Color(0x474747),
                new Color(0x484848),
                new Color(0x494949),
                new Color(0x4a4a4a),
                new Color(0x4b4b4b),
                new Color(0x4c4c4c),
                new Color(0x4d4d4d),
                new Color(0x4e4e4e),
                new Color(0x4f4f4f),
                new Color(0x505050),
                new Color(0x515151),
                new Color(0x525252),
                new Color(0x535353),
                new Color(0x545454),
                new Color(0x555555),
                new Color(0x565656),
                new Color(0x575757),
                new Color(0x585858),
                new Color(0x595959),
                new Color(0x5a5a5a),
                new Color(0x5b5b5b),
                new Color(0x5c5c5c),
                new Color(0x5d5d5d),
                new Color(0x5e5e5e),
                new Color(0x5f5f5f),
                new Color(0x606060),
                new Color(0x616161),
                new Color(0x626262),
                new Color(0x636363),
                new Color(0x646464),
                new Color(0x656565),
                new Color(0x666666),
                new Color(0x676767),
                new Color(0x686868),
                new Color(0x696969),
                new Color(0x6a6a6a),
                new Color(0x6b6b6b),
                new Color(0x6c6c6c),
                new Color(0x6d6d6d),
                new Color(0x6e6e6e),
                new Color(0x6f6f6f),
                new Color(0x707070),
                new Color(0x717171),
                new Color(0x727272),
                new Color(0x737373),
                new Color(0x747474),
                new Color(0x757575),
                new Color(0x767676),
                new Color(0x777777),
                new Color(0x787878),
                new Color(0x797979),
                new Color(0x7a7a7a),
                new Color(0x7b7b7b),
                new Color(0x7c7c7c),
                new Color(0x7d7d7d),
                new Color(0x7e7e7e),
                new Color(0x7f7f7f),
                new Color(0x808080),
                new Color(0x818181),
                new Color(0x828282),
                new Color(0x838383),
                new Color(0x848484),
                new Color(0x858585),
                new Color(0x868686),
                new Color(0x878787),
                new Color(0x888888),
                new Color(0x898989),
                new Color(0x8a8a8a),
                new Color(0x8b8b8b),
                new Color(0x8c8c8c),
                new Color(0x8d8d8d),
                new Color(0x8e8e8e),
                new Color(0x8f8f8f),
                new Color(0x909090),
                new Color(0x919191),
                new Color(0x929292),
                new Color(0x939393),
                new Color(0x949494),
                new Color(0x959595),
                new Color(0x969696),
                new Color(0x979797),
                new Color(0x989898),
                new Color(0x999999),
                new Color(0x9a9a9a),
                new Color(0x9b9b9b),
                new Color(0x9c9c9c),
                new Color(0x9d9d9d),
                new Color(0x9e9e9e),
                new Color(0x9f9f9f),
                new Color(0xa0a0a0),
                new Color(0xa1a1a1),
                new Color(0xa2a2a2),
                new Color(0xa3a3a3),
                new Color(0xa4a4a4),
                new Color(0xa5a5a5),
                new Color(0xa6a6a6),
                new Color(0xa7a7a7),
                new Color(0xa8a8a8),
                new Color(0xa9a9a9),
                new Color(0xaaaaaa),
                new Color(0xababab),
                new Color(0xacacac),
                new Color(0xadadad),
                new Color(0xaeaeae),
                new Color(0xafafaf),
                new Color(0xb0b0b0),
                new Color(0xb1b1b1),
                new Color(0xb2b2b2),
                new Color(0xb3b3b3),
                new Color(0xb4b4b4),
                new Color(0xb5b5b5),
                new Color(0xb6b6b6),
                new Color(0xb7b7b7),
                new Color(0xb8b8b8),
                new Color(0xb9b9b9),
                new Color(0xbababa),
                new Color(0xbbbbbb),
                new Color(0xbcbcbc),
                new Color(0xbdbdbd),
                new Color(0xbebebe),
                new Color(0xbfbfbf),
                new Color(0xc0c0c0),
                new Color(0xc1c1c1),
                new Color(0xc2c2c2),
                new Color(0xc3c3c3),
                new Color(0xc4c4c4),
                new Color(0xc5c5c5),
                new Color(0xc6c6c6),
                new Color(0xc7c7c7),
                new Color(0xc8c8c8),
                new Color(0xc9c9c9),
                new Color(0xcacaca),
                new Color(0xcbcbcb),
                new Color(0xcccccc),
                new Color(0xcdcdcd),
                new Color(0xcecece),
                new Color(0xcfcfcf),
                new Color(0xd0d0d0),
                new Color(0xd1d1d1),
                new Color(0xd2d2d2),
                new Color(0xd3d3d3),
                new Color(0xd4d4d4),
                new Color(0xd5d5d5),
                new Color(0xd6d6d6),
                new Color(0xd7d7d7),
                new Color(0xd8d8d8),
                new Color(0xd9d9d9),
                new Color(0xdadada),
                new Color(0xdbdbdb),
                new Color(0xdcdcdc),
                new Color(0xdddddd),
                new Color(0xdedede),
                new Color(0xdfdfdf),
                new Color(0xe0e0e0),
                new Color(0xe1e1e1),
                new Color(0xe2e2e2),
                new Color(0xe3e3e3),
                new Color(0xe4e4e4),
                new Color(0xe5e5e5),
                new Color(0xe6e6e6),
                new Color(0xe7e7e7),
                new Color(0xe8e8e8),
                new Color(0xe9e9e9),
                new Color(0xeaeaea),
                new Color(0xebebeb),
                new Color(0xececec),
                new Color(0xededed),
                new Color(0xeeeeee),
                new Color(0xefefef),
                new Color(0xf0f0f0),
                new Color(0xf1f1f1),
                new Color(0xf2f2f2),
                new Color(0xf3f3f3),
                new Color(0xf4f4f4),
                new Color(0xf5f5f5),
                new Color(0xf6f6f6),
                new Color(0xf7f7f7),
                new Color(0xf8f8f8),
                new Color(0xf9f9f9),
                new Color(0xfafafa),
                new Color(0xfbfbfb),
                new Color(0xfcfcfc),
                new Color(0xfdfdfd),
                new Color(0xfefefe),
                new Color(0xffffff),
    };
    private static final Colourmap z_grayscale_rev = new Colourmap(greyScaleZAxisRev,"Greyscale - Reverse");
    public static final Color[] greyScaleZAxis ={
        new Color(0xffffff),
                new Color(0xfefefe),
                new Color(0xfdfdfd),
                new Color(0xfcfcfc),
                new Color(0xfbfbfb),
                new Color(0xfafafa),
                new Color(0xf9f9f9),
                new Color(0xf8f8f8),
                new Color(0xf7f7f7),
                new Color(0xf6f6f6),
                new Color(0xf5f5f5),
                new Color(0xf4f4f4),
                new Color(0xf3f3f3),
                new Color(0xf2f2f2),
                new Color(0xf1f1f1),
                new Color(0xf0f0f0),
                new Color(0xefefef),
                new Color(0xeeeeee),
                new Color(0xededed),
                new Color(0xececec),
                new Color(0xebebeb),
                new Color(0xeaeaea),
                new Color(0xe9e9e9),
                new Color(0xe8e8e8),
                new Color(0xe7e7e7),
                new Color(0xe6e6e6),
                new Color(0xe5e5e5),
                new Color(0xe4e4e4),
                new Color(0xe3e3e3),
                new Color(0xe2e2e2),
                new Color(0xe1e1e1),
                new Color(0xe0e0e0),
                new Color(0xdfdfdf),
                new Color(0xdedede),
                new Color(0xdddddd),
                new Color(0xdcdcdc),
                new Color(0xdbdbdb),
                new Color(0xdadada),
                new Color(0xd9d9d9),
                new Color(0xd8d8d8),
                new Color(0xd7d7d7),
                new Color(0xd6d6d6),
                new Color(0xd5d5d5),
                new Color(0xd4d4d4),
                new Color(0xd3d3d3),
                new Color(0xd2d2d2),
                new Color(0xd1d1d1),
                new Color(0xd0d0d0),
                new Color(0xcfcfcf),
                new Color(0xcecece),
                new Color(0xcdcdcd),
                new Color(0xcccccc),
                new Color(0xcbcbcb),
                new Color(0xcacaca),
                new Color(0xc9c9c9),
                new Color(0xc8c8c8),
                new Color(0xc7c7c7),
                new Color(0xc6c6c6),
                new Color(0xc5c5c5),
                new Color(0xc4c4c4),
                new Color(0xc3c3c3),
                new Color(0xc2c2c2),
                new Color(0xc1c1c1),
                new Color(0xc0c0c0),
                new Color(0xbfbfbf),
                new Color(0xbebebe),
                new Color(0xbdbdbd),
                new Color(0xbcbcbc),
                new Color(0xbbbbbb),
                new Color(0xbababa),
                new Color(0xb9b9b9),
                new Color(0xb8b8b8),
                new Color(0xb7b7b7),
                new Color(0xb6b6b6),
                new Color(0xb5b5b5),
                new Color(0xb4b4b4),
                new Color(0xb3b3b3),
                new Color(0xb2b2b2),
                new Color(0xb1b1b1),
                new Color(0xb0b0b0),
                new Color(0xafafaf),
                new Color(0xaeaeae),
                new Color(0xadadad),
                new Color(0xacacac),
                new Color(0xababab),
                new Color(0xaaaaaa),
                new Color(0xa9a9a9),
                new Color(0xa8a8a8),
                new Color(0xa7a7a7),
                new Color(0xa6a6a6),
                new Color(0xa5a5a5),
                new Color(0xa4a4a4),
                new Color(0xa3a3a3),
                new Color(0xa2a2a2),
                new Color(0xa1a1a1),
                new Color(0xa0a0a0),
                new Color(0x9f9f9f),
                new Color(0x9e9e9e),
                new Color(0x9d9d9d),
                new Color(0x9c9c9c),
                new Color(0x9b9b9b),
                new Color(0x9a9a9a),
                new Color(0x999999),
                new Color(0x989898),
                new Color(0x979797),
                new Color(0x969696),
                new Color(0x959595),
                new Color(0x949494),
                new Color(0x939393),
                new Color(0x929292),
                new Color(0x919191),
                new Color(0x909090),
                new Color(0x8f8f8f),
                new Color(0x8e8e8e),
                new Color(0x8d8d8d),
                new Color(0x8c8c8c),
                new Color(0x8b8b8b),
                new Color(0x8a8a8a),
                new Color(0x898989),
                new Color(0x888888),
                new Color(0x878787),
                new Color(0x868686),
                new Color(0x858585),
                new Color(0x848484),
                new Color(0x838383),
                new Color(0x828282),
                new Color(0x818181),
                new Color(0x808080),
                new Color(0x7f7f7f),
                new Color(0x7e7e7e),
                new Color(0x7d7d7d),
                new Color(0x7c7c7c),
                new Color(0x7b7b7b),
                new Color(0x7a7a7a),
                new Color(0x797979),
                new Color(0x787878),
                new Color(0x777777),
                new Color(0x767676),
                new Color(0x757575),
                new Color(0x747474),
                new Color(0x737373),
                new Color(0x727272),
                new Color(0x717171),
                new Color(0x707070),
                new Color(0x6f6f6f),
                new Color(0x6e6e6e),
                new Color(0x6d6d6d),
                new Color(0x6c6c6c),
                new Color(0x6b6b6b),
                new Color(0x6a6a6a),
                new Color(0x696969),
                new Color(0x686868),
                new Color(0x676767),
                new Color(0x666666),
                new Color(0x656565),
                new Color(0x646464),
                new Color(0x636363),
                new Color(0x626262),
                new Color(0x616161),
                new Color(0x606060),
                new Color(0x5f5f5f),
                new Color(0x5e5e5e),
                new Color(0x5d5d5d),
                new Color(0x5c5c5c),
                new Color(0x5b5b5b),
                new Color(0x5a5a5a),
                new Color(0x595959),
                new Color(0x585858),
                new Color(0x575757),
                new Color(0x565656),
                new Color(0x555555),
                new Color(0x545454),
                new Color(0x535353),
                new Color(0x525252),
                new Color(0x515151),
                new Color(0x505050),
                new Color(0x4f4f4f),
                new Color(0x4e4e4e),
                new Color(0x4d4d4d),
                new Color(0x4c4c4c),
                new Color(0x4b4b4b),
                new Color(0x4a4a4a),
                new Color(0x494949),
                new Color(0x484848),
                new Color(0x474747),
                new Color(0x464646),
                new Color(0x454545),
                new Color(0x444444),
                new Color(0x434343),
                new Color(0x424242),
                new Color(0x414141),
                new Color(0x404040),
                new Color(0x3f3f3f),
                new Color(0x3e3e3e),
                new Color(0x3d3d3d),
                new Color(0x3c3c3c),
                new Color(0x3b3b3b),
                new Color(0x3a3a3a),
                new Color(0x393939),
                new Color(0x383838),
                new Color(0x373737),
                new Color(0x363636),
                new Color(0x353535),
                new Color(0x343434),
                new Color(0x333333),
                new Color(0x323232),
                new Color(0x313131),
                new Color(0x303030),
                new Color(0x2f2f2f),
                new Color(0x2e2e2e),
                new Color(0x2d2d2d),
                new Color(0x2c2c2c),
                new Color(0x2b2b2b),
                new Color(0x2a2a2a),
                new Color(0x292929),
                new Color(0x282828),
                new Color(0x272727),
                new Color(0x262626),
                new Color(0x252525),
                new Color(0x242424),
                new Color(0x232323),
                new Color(0x222222),
                new Color(0x212121),
                new Color(0x202020),
                new Color(0x1f1f1f),
                new Color(0x1e1e1e),
                new Color(0x1d1d1d),
                new Color(0x1c1c1c),
                new Color(0x1b1b1b),
                new Color(0x1a1a1a),
                new Color(0x191919),
                new Color(0x181818),
                new Color(0x171717),
                new Color(0x161616),
                new Color(0x151515),
                new Color(0x141414),
                new Color(0x131313),
                new Color(0x121212),
                new Color(0x111111),
                new Color(0x101010),
                new Color(0x0f0f0f),
                new Color(0x0e0e0e),
                new Color(0x0d0d0d),
                new Color(0x0c0c0c),
                new Color(0x0b0b0b),
                new Color(0x0a0a0a),
                new Color(0x090909),
                new Color(0x080808),
                new Color(0x070707),
                new Color(0x060606),
                new Color(0x050505),
                new Color(0x040404),
                new Color(0x030303),
                new Color(0x020202),
                new Color(0x010101),
                new Color(0x000000),
    };
    
    private static final Colourmap z_grayscale = new Colourmap(greyScaleZAxis,"Greyscale");
    
    public static final Color[] colourZaxis_rev={
        new Color(0x0000ff),
                new Color(0x0002fd),
                new Color(0x0004fb),
                new Color(0x0006f9),
                new Color(0x0008f7),
                new Color(0x000af5),
                new Color(0x000cf3),
                new Color(0x000ef1),
                new Color(0x0010ef),
                new Color(0x0012ed),
                new Color(0x0014eb),
                new Color(0x0016e9),
                new Color(0x0018e7),
                new Color(0x001ae5),
                new Color(0x001ce3),
                new Color(0x001ee1),
                new Color(0x0020df),
                new Color(0x0022dd),
                new Color(0x0024db),
                new Color(0x0026d9),
                new Color(0x0028d7),
                new Color(0x002ad5),
                new Color(0x002cd3),
                new Color(0x002ed1),
                new Color(0x0030cf),
                new Color(0x0032cd),
                new Color(0x0034cb),
                new Color(0x0036c9),
                new Color(0x0038c7),
                new Color(0x003ac5),
                new Color(0x003cc3),
                new Color(0x003ec1),
                new Color(0x0040bf),
                new Color(0x0042bd),
                new Color(0x0044bb),
                new Color(0x0046b9),
                new Color(0x0048b7),
                new Color(0x004ab5),
                new Color(0x004cb3),
                new Color(0x004eb1),
                new Color(0x0050af),
                new Color(0x0052ad),
                new Color(0x0054ab),
                new Color(0x0056a9),
                new Color(0x0058a7),
                new Color(0x005aa5),
                new Color(0x005ca3),
                new Color(0x005ea1),
                new Color(0x00609f),
                new Color(0x00629d),
                new Color(0x00649b),
                new Color(0x006699),
                new Color(0x006897),
                new Color(0x006a95),
                new Color(0x006c93),
                new Color(0x006e91),
                new Color(0x00708f),
                new Color(0x00728d),
                new Color(0x00748b),
                new Color(0x007689),
                new Color(0x007887),
                new Color(0x007a85),
                new Color(0x007c83),
                new Color(0x007e81),
                new Color(0x00807f),
                new Color(0x00827d),
                new Color(0x00847b),
                new Color(0x008679),
                new Color(0x008877),
                new Color(0x008a75),
                new Color(0x008c73),
                new Color(0x008e71),
                new Color(0x00906f),
                new Color(0x00926d),
                new Color(0x00946b),
                new Color(0x009669),
                new Color(0x009867),
                new Color(0x009a65),
                new Color(0x009c63),
                new Color(0x009e61),
                new Color(0x00a05f),
                new Color(0x00a25d),
                new Color(0x00a45b),
                new Color(0x00a659),
                new Color(0x00a857),
                new Color(0x00aa55),
                new Color(0x00ac53),
                new Color(0x00ae51),
                new Color(0x00b04f),
                new Color(0x00b24d),
                new Color(0x00b44b),
                new Color(0x00b649),
                new Color(0x00b847),
                new Color(0x00ba45),
                new Color(0x00bc43),
                new Color(0x00be41),
                new Color(0x00c03f),
                new Color(0x00c23d),
                new Color(0x00c43b),
                new Color(0x00c639),
                new Color(0x00c837),
                new Color(0x00ca35),
                new Color(0x00cc33),
                new Color(0x00ce31),
                new Color(0x00d02f),
                new Color(0x00d22d),
                new Color(0x00d42b),
                new Color(0x00d629),
                new Color(0x00d827),
                new Color(0x00da25),
                new Color(0x00dc23),
                new Color(0x00de21),
                new Color(0x00e01f),
                new Color(0x00e21d),
                new Color(0x00e41b),
                new Color(0x00e619),
                new Color(0x00e817),
                new Color(0x00ea15),
                new Color(0x00ec13),
                new Color(0x00ee11),
                new Color(0x00f00f),
                new Color(0x00f20d),
                new Color(0x00f40b),
                new Color(0x00f609),
                new Color(0x00f807),
                new Color(0x00fa05),
                new Color(0x00fc03),
                new Color(0x00fe01),
                new Color(0x00ff00),
                new Color(0x02fd00),
                new Color(0x04fb00),
                new Color(0x06f900),
                new Color(0x08f700),
                new Color(0x0af500),
                new Color(0x0cf300),
                new Color(0x0ef100),
                new Color(0x10ef00),
                new Color(0x12ed00),
                new Color(0x14eb00),
                new Color(0x16e900),
                new Color(0x18e700),
                new Color(0x1ae500),
                new Color(0x1ce300),
                new Color(0x1ee100),
                new Color(0x20df00),
                new Color(0x22dd00),
                new Color(0x24db00),
                new Color(0x26d900),
                new Color(0x28d700),
                new Color(0x2ad500),
                new Color(0x2cd300),
                new Color(0x2ed100),
                new Color(0x30cf00),
                new Color(0x32cd00),
                new Color(0x34cb00),
                new Color(0x36c900),
                new Color(0x38c700),
                new Color(0x3ac500),
                new Color(0x3cc300),
                new Color(0x3ec100),
                new Color(0x40bf00),
                new Color(0x42bd00),
                new Color(0x44bb00),
                new Color(0x46b900),
                new Color(0x48b700),
                new Color(0x4ab500),
                new Color(0x4cb300),
                new Color(0x4eb100),
                new Color(0x50af00),
                new Color(0x52ad00),
                new Color(0x54ab00),
                new Color(0x56a900),
                new Color(0x58a700),
                new Color(0x5aa500),
                new Color(0x5ca300),
                new Color(0x5ea100),
                new Color(0x609f00),
                new Color(0x629d00),
                new Color(0x649b00),
                new Color(0x669900),
                new Color(0x689700),
                new Color(0x6a9500),
                new Color(0x6c9300),
                new Color(0x6e9100),
                new Color(0x708f00),
                new Color(0x728d00),
                new Color(0x748b00),
                new Color(0x768900),
                new Color(0x788700),
                new Color(0x7a8500),
                new Color(0x7c8300),
                new Color(0x7e8100),
                new Color(0x807f00),
                new Color(0x827d00),
                new Color(0x847b00),
                new Color(0x867900),
                new Color(0x887700),
                new Color(0x8a7500),
                new Color(0x8c7300),
                new Color(0x8e7100),
                new Color(0x906f00),
                new Color(0x926d00),
                new Color(0x946b00),
                new Color(0x966900),
                new Color(0x986700),
                new Color(0x9a6500),
                new Color(0x9c6300),
                new Color(0x9e6100),
                new Color(0xa05f00),
                new Color(0xa25d00),
                new Color(0xa45b00),
                new Color(0xa65900),
                new Color(0xa85700),
                new Color(0xaa5500),
                new Color(0xac5300),
                new Color(0xae5100),
                new Color(0xb04f00),
                new Color(0xb24d00),
                new Color(0xb44b00),
                new Color(0xb64900),
                new Color(0xb84700),
                new Color(0xba4500),
                new Color(0xbc4300),
                new Color(0xbe4100),
                new Color(0xc03f00),
                new Color(0xc23d00),
                new Color(0xc43b00),
                new Color(0xc63900),
                new Color(0xc83700),
                new Color(0xca3500),
                new Color(0xcc3300),
                new Color(0xce3100),
                new Color(0xd02f00),
                new Color(0xd22d00),
                new Color(0xd42b00),
                new Color(0xd62900),
                new Color(0xd82700),
                new Color(0xda2500),
                new Color(0xdc2300),
                new Color(0xde2100),
                new Color(0xe01f00),
                new Color(0xe21d00),
                new Color(0xe41b00),
                new Color(0xe61900),
                new Color(0xe81700),
                new Color(0xea1500),
                new Color(0xec1300),
                new Color(0xee1100),
                new Color(0xf00f00),
                new Color(0xf20d00),
                new Color(0xf40b00),
                new Color(0xf60900),
                new Color(0xf80700),
                new Color(0xfa0500),
                new Color(0xfc0300),
                new Color(0xfe0100),
    };
    
    private static final Colourmap z_colour_rev = new Colourmap(colourZaxis_rev,"Colour Wheel - reverse");
    
    private static Color[] colourZaxis = {
        new Color(0xff0000),
                new Color(0xfd0200),
                new Color(0xfb0400),
                new Color(0xf90600),
                new Color(0xf70800),
                new Color(0xf50a00),
                new Color(0xf30c00),
                new Color(0xf10e00),
                new Color(0xef1000),
                new Color(0xed1200),
                new Color(0xeb1400),
                new Color(0xe91600),
                new Color(0xe71800),
                new Color(0xe51a00),
                new Color(0xe31c00),
                new Color(0xe11e00),
                new Color(0xdf2000),
                new Color(0xdd2200),
                new Color(0xdb2400),
                new Color(0xd92600),
                new Color(0xd72800),
                new Color(0xd52a00),
                new Color(0xd32c00),
                new Color(0xd12e00),
                new Color(0xcf3000),
                new Color(0xcd3200),
                new Color(0xcb3400),
                new Color(0xc93600),
                new Color(0xc73800),
                new Color(0xc53a00),
                new Color(0xc33c00),
                new Color(0xc13e00),
                new Color(0xbf4000),
                new Color(0xbd4200),
                new Color(0xbb4400),
                new Color(0xb94600),
                new Color(0xb74800),
                new Color(0xb54a00),
                new Color(0xb34c00),
                new Color(0xb14e00),
                new Color(0xaf5000),
                new Color(0xad5200),
                new Color(0xab5400),
                new Color(0xa95600),
                new Color(0xa75800),
                new Color(0xa55a00),
                new Color(0xa35c00),
                new Color(0xa15e00),
                new Color(0x9f6000),
                new Color(0x9d6200),
                new Color(0x9b6400),
                new Color(0x996600),
                new Color(0x976800),
                new Color(0x956a00),
                new Color(0x936c00),
                new Color(0x916e00),
                new Color(0x8f7000),
                new Color(0x8d7200),
                new Color(0x8b7400),
                new Color(0x897600),
                new Color(0x877800),
                new Color(0x857a00),
                new Color(0x837c00),
                new Color(0x817e00),
                new Color(0x7f8000),
                new Color(0x7d8200),
                new Color(0x7b8400),
                new Color(0x798600),
                new Color(0x778800),
                new Color(0x758a00),
                new Color(0x738c00),
                new Color(0x718e00),
                new Color(0x6f9000),
                new Color(0x6d9200),
                new Color(0x6b9400),
                new Color(0x699600),
                new Color(0x679800),
                new Color(0x659a00),
                new Color(0x639c00),
                new Color(0x619e00),
                new Color(0x5fa000),
                new Color(0x5da200),
                new Color(0x5ba400),
                new Color(0x59a600),
                new Color(0x57a800),
                new Color(0x55aa00),
                new Color(0x53ac00),
                new Color(0x51ae00),
                new Color(0x4fb000),
                new Color(0x4db200),
                new Color(0x4bb400),
                new Color(0x49b600),
                new Color(0x47b800),
                new Color(0x45ba00),
                new Color(0x43bc00),
                new Color(0x41be00),
                new Color(0x3fc000),
                new Color(0x3dc200),
                new Color(0x3bc400),
                new Color(0x39c600),
                new Color(0x37c800),
                new Color(0x35ca00),
                new Color(0x33cc00),
                new Color(0x31ce00),
                new Color(0x2fd000),
                new Color(0x2dd200),
                new Color(0x2bd400),
                new Color(0x29d600),
                new Color(0x27d800),
                new Color(0x25da00),
                new Color(0x23dc00),
                new Color(0x21de00),
                new Color(0x1fe000),
                new Color(0x1de200),
                new Color(0x1be400),
                new Color(0x19e600),
                new Color(0x17e800),
                new Color(0x15ea00),
                new Color(0x13ec00),
                new Color(0x11ee00),
                new Color(0x0ff000),
                new Color(0x0df200),
                new Color(0x0bf400),
                new Color(0x09f600),
                new Color(0x07f800),
                new Color(0x05fa00),
                new Color(0x03fc00),
                new Color(0x01fe00),
                new Color(0x00ff00),
                new Color(0x00fd02),
                new Color(0x00fb04),
                new Color(0x00f906),
                new Color(0x00f708),
                new Color(0x00f50a),
                new Color(0x00f30c),
                new Color(0x00f10e),
                new Color(0x00ef10),
                new Color(0x00ed12),
                new Color(0x00eb14),
                new Color(0x00e916),
                new Color(0x00e718),
                new Color(0x00e51a),
                new Color(0x00e31c),
                new Color(0x00e11e),
                new Color(0x00df20),
                new Color(0x00dd22),
                new Color(0x00db24),
                new Color(0x00d926),
                new Color(0x00d728),
                new Color(0x00d52a),
                new Color(0x00d32c),
                new Color(0x00d12e),
                new Color(0x00cf30),
                new Color(0x00cd32),
                new Color(0x00cb34),
                new Color(0x00c936),
                new Color(0x00c738),
                new Color(0x00c53a),
                new Color(0x00c33c),
                new Color(0x00c13e),
                new Color(0x00bf40),
                new Color(0x00bd42),
                new Color(0x00bb44),
                new Color(0x00b946),
                new Color(0x00b748),
                new Color(0x00b54a),
                new Color(0x00b34c),
                new Color(0x00b14e),
                new Color(0x00af50),
                new Color(0x00ad52),
                new Color(0x00ab54),
                new Color(0x00a956),
                new Color(0x00a758),
                new Color(0x00a55a),
                new Color(0x00a35c),
                new Color(0x00a15e),
                new Color(0x009f60),
                new Color(0x009d62),
                new Color(0x009b64),
                new Color(0x009966),
                new Color(0x009768),
                new Color(0x00956a),
                new Color(0x00936c),
                new Color(0x00916e),
                new Color(0x008f70),
                new Color(0x008d72),
                new Color(0x008b74),
                new Color(0x008976),
                new Color(0x008778),
                new Color(0x00857a),
                new Color(0x00837c),
                new Color(0x00817e),
                new Color(0x007f80),
                new Color(0x007d82),
                new Color(0x007b84),
                new Color(0x007986),
                new Color(0x007788),
                new Color(0x00758a),
                new Color(0x00738c),
                new Color(0x00718e),
                new Color(0x006f90),
                new Color(0x006d92),
                new Color(0x006b94),
                new Color(0x006996),
                new Color(0x006798),
                new Color(0x00659a),
                new Color(0x00639c),
                new Color(0x00619e),
                new Color(0x005fa0),
                new Color(0x005da2),
                new Color(0x005ba4),
                new Color(0x0059a6),
                new Color(0x0057a8),
                new Color(0x0055aa),
                new Color(0x0053ac),
                new Color(0x0051ae),
                new Color(0x004fb0),
                new Color(0x004db2),
                new Color(0x004bb4),
                new Color(0x0049b6),
                new Color(0x0047b8),
                new Color(0x0045ba),
                new Color(0x0043bc),
                new Color(0x0041be),
                new Color(0x003fc0),
                new Color(0x003dc2),
                new Color(0x003bc4),
                new Color(0x0039c6),
                new Color(0x0037c8),
                new Color(0x0035ca),
                new Color(0x0033cc),
                new Color(0x0031ce),
                new Color(0x002fd0),
                new Color(0x002dd2),
                new Color(0x002bd4),
                new Color(0x0029d6),
                new Color(0x0027d8),
                new Color(0x0025da),
                new Color(0x0023dc),
                new Color(0x0021de),
                new Color(0x001fe0),
                new Color(0x001de2),
                new Color(0x001be4),
                new Color(0x0019e6),
                new Color(0x0017e8),
                new Color(0x0015ea),
                new Color(0x0013ec),
                new Color(0x0011ee),
                new Color(0x000ff0),
                new Color(0x000df2),
                new Color(0x000bf4),
                new Color(0x0009f6),
                new Color(0x0007f8),
                new Color(0x0005fa),
                new Color(0x0003fc),
                new Color(0x0001fe),
    };
    
    private static final Colourmap z_colour = new Colourmap(colourZaxis,"Colour Wheel");
    
    public static final Colourmap[] zMaps = {z_grayscale,z_grayscale_rev,z_colour,z_colour_rev};
    
    public static  Color[] stdColmap = {
        /*0:std*/     new Color(0xff0000),   // red
                /*1:acc*/     new Color(0x0000ff),   // blue
                /*2:lng*/     new Color(0x00ff00),   // green
                /*3:???*/     new Color(0x45ab1f),   // black
                /*4:???*/     new Color(0xffa500),   // orange
                /*5:std*/     new Color(0xff0000),   // red
                /*6:acc*/     new Color(0x0000ff),   // blue
                /*7:lng*/     new Color(0x00ff00),   // green
                /*8:???*/     new Color(0x45ab1f),   // black
                /*9:???*/     new Color(0xffa500),   // orange
                /*10:known*/  new Color(0xFF9D1F),   // orange
                /*11:class1*/ new Color(0x000000),   // cyan-ish
                /*12:class2*/ new Color(0x808080),   // cadetblue4
                /*13:class3*/ new Color(0xc0c0c0),   // dark green-ish
                /*14:dud*/     new Color(0xc0c0c0),   // coral
                /*15:???*/     new Color(0xc0c0c0),   // cadetblue4
                /*16:Last*/     new Color(0x40E0D0),   // coral
                /*17:???*/     new Color(0x45ab1f),   // dark green-ish
                /*18:???*/     new Color(0x90422d),   // sienna-ish
                /*19:???*/     new Color(0xa0a0a0),   // grey-ish
    };
    private static final Colourmap std_defaultColmap = new Colourmap(stdColmap,"Standard Cols");
    
    
    
    
    public static Colourmap defaultSTDColmap = std_defaultColmap;
    public static Colourmap defaultZColmap = z_colour;
    public static Colourmap defaultGreyColmap = gray_quadratic;
    private static Colourmap zCol = defaultZColmap;
    private static Colourmap gCol = defaultGreyColmap;
}
